@@ -1,7 +1,7 @@
 /*
-  $Id: tracks.c,v 1.3 2005/07/07 06:58:48 rocky Exp $
+  $Id: tracks.c,v 1.5 2006/01/22 08:17:40 rocky Exp $
 
-  Copyright (C) 2003, 2004, 2005 Rocky Bernstein <rocky@panix.com>
+  Copyright (C) 2003, 2004, 2005, 2006 Rocky Bernstein <rocky@panix.com>
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,9 @@
 # include "config.h"
 #endif
 #include <stdio.h>
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
 #include <cdio/cdio.h>
 int
 main(int argc, const char *argv[])
@@ -43,7 +45,8 @@ main(int argc, const char *argv[])
   i_tracks      = cdio_get_num_tracks(p_cdio);
   i_first_track = i = cdio_get_first_track_num(p_cdio);
 
-  printf("CD-ROM Track List (%i - %i)\n", i_first_track, i_tracks);
+  printf("CD-ROM Track List (%i - %i)\n", i_first_track, 
+	 i_first_track+i_tracks-1);
 
   printf("  #:  LSN\n");
   
