@@ -1,8 +1,8 @@
 /*
-    $Id: iso9660.h,v 1.74 2005/02/22 10:42:50 rocky Exp $
+    $Id: iso9660.h,v 1.90 2006/03/18 02:35:07 rocky Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
-    Copyright (C) 2003, 2004, 2005 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2003, 2004, 2005, 2006 Rocky Bernstein <rocky@panix.com>
 
     See also iso9660.h by Eric Youngdale (1993).
 
@@ -21,12 +21,13 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301 USA.
 */
 /*!
  * \file iso9660.h 
  *
- * \brief The top-level interface eader for libiso9660: the ISO-9660
+ * \brief The top-level interface header for libiso9660: the ISO-9660
  * filesystem library; applications include this.
  *
  * See also the ISO-9660 specification. The freely available European
@@ -41,6 +42,7 @@
 
 #include <cdio/cdio.h>
 #include <cdio/ds.h>
+#include <cdio/posix.h>
 
 /** \brief ISO 9660 Integer and Character types 
 
@@ -72,24 +74,26 @@ typedef char     dchar_t;  /*! See section 7.4.1 */
 #define MIN_TRACK_SIZE 4*75
 #define MIN_ISO_SIZE MIN_TRACK_SIZE
 
-/*! An enumeration for some of the ISO_* #defines below. This isn't
-    really an enumeration one would really use in a program it is here
-    to be helpful in debuggers where wants just to refer to the
-    ISO_*_ names and get something.
+/*! The below isn't really an enumeration one would really use in a
+    program; things are done this way so that in a debugger one can to
+    refer to the enumeration value names such as in a debugger
+    expression and get something. With the more common a \#define
+    mechanism, the name/value assocation is lost at run time.
   */
 extern enum iso_enum1_s {
-  ISO_PVD_SECTOR      =   16, /**< Sector of Primary Volume Descriptor */
-  ISO_EVD_SECTOR      =   17, /**< Sector of End Volume Descriptor */
-  LEN_ISONAME         =   31, /**<  size in bytes of the filename 
-                                 portion + null byte */
+  ISO_PVD_SECTOR      =   16, /**< Sector of Primary Volume Descriptor. */
+  ISO_EVD_SECTOR      =   17, /**< Sector of End Volume Descriptor. */
+  LEN_ISONAME         =   31, /**< Size in bytes of the filename 
+                                 portion + null byte. */
   ISO_MAX_SYSTEM_ID   =   32, /**< Maximum number of characters in a system 
                                  id. */
-  MAX_ISONAME         =   37, /**< size in bytes of the filename 
-                                 portion + null byte */
+  MAX_ISONAME         =   37, /**< Size in bytes of the filename 
+                                 portion + null byte. */
   ISO_MAX_PREPARER_ID =  128, /**< Maximum number of characters in a
                                  preparer id. */
-  MAX_ISOPATHNAME     =  255,
-  ISO_BLOCKSIZE       = 2048
+  MAX_ISOPATHNAME     =  255, /**< Maximum number of characters in the
+                                 entire ISO 9660 filename. */
+  ISO_BLOCKSIZE       = 2048  /**< Number of bytes in an ISO 9660 block. */
 
 } iso_enums1;
 
@@ -98,24 +102,27 @@ extern enum iso_enum1_s {
     to be helpful in debuggers where wants just to refer to the
     ISO_*_ names and get something.
   */
+
+/**! ISO 9660 directory flags. */
 extern enum iso_flag_enum_s {
   ISO_FILE	      =   0,   /**<  Not really a flag...	         */
   ISO_EXISTENCE	      =   1,   /**< Do not make existence known (hidden) */
   ISO_DIRECTORY	      =   2,   /**< This file is a directory		 */
   ISO_ASSOCIATED      =   4,   /**< This file is an associated file	 */
   ISO_RECORD	      =   8,   /**< Record format in extended attr. != 0 */
-  ISO_PROECTION	      =  16,   /**< No read/execute perm. in ext. attr.  */
+  ISO_PROTECTION      =  16,   /**< No read/execute perm. in ext. attr.  */
   ISO_DRESERVED1      =  32,   /**<, Reserved bit 5			 */
   ISO_DRESERVED2      =  64,   /**<, Reserved bit 6			 */
   ISO_MULTIEXTENT     = 128,   /**< Not final entry of a mult. ext. file */
 } iso_flag_enums;
 
+/**! Volume descriptor types */
 extern enum iso_vd_enum_s {
-  ISO_VD_BOOT_RECORD  =  0,  /**< CD is bootable */
-  ISO_VD_PRIMARY      =  1,  /**< Is in any ISO-9660 */
-  ISO_VD_SUPPLEMENARY =  2,  /**< Used by Joliet, for example */
-  ISO_VD_PARITION     =  3,  /**< Indicates a partition of a CD */
-  ISO_VD_END	      = 255
+  ISO_VD_BOOT_RECORD   =  0,  /**< CD is bootable */
+  ISO_VD_PRIMARY       =  1,  /**< Is in any ISO-9660 */
+  ISO_VD_SUPPLEMENTARY =  2,  /**< Used by Joliet, for example */
+  ISO_VD_PARITION      =  3,  /**< Indicates a partition of a CD */
+  ISO_VD_END	       = 255
 } iso_vd_enums;
 
   
@@ -135,58 +142,18 @@ extern enum iso_vd_enum_s {
 
 */
 
-
-/*! \brief size in bytes of the filename portion + null byte */
-#define LEN_ISONAME      31
-
-/*! \brief Maximum number of characters in the entire ISO 9660 filename. */
-#define MAX_ISONAME      37
-
-/*! \brief Maximum number of characters in the entire ISO 9660 filename. */
-#define MAX_ISOPATHNAME 255
-
-/*! \brief Maximum number of characters in a preparer id. */
-#define ISO_MAX_PREPARER_ID 128
-
 /*! \brief Maximum number of characters in a publisher id. */
 #define ISO_MAX_PUBLISHER_ID 128
 
 /*! \brief Maximum number of characters in an application id. */
 #define ISO_MAX_APPLICATION_ID 128
 
-/*! \brief Maximum number of characters in a system id. */
-#define ISO_MAX_SYSTEM_ID 32
-
 /*! \brief Maximum number of characters in a volume id. */
 #define ISO_MAX_VOLUME_ID 32
 
 /*! \brief Maximum number of characters in a volume-set id. */
 #define ISO_MAX_VOLUMESET_ID 128
 
-/**! ISO 9660 directory flags. */
-#define	ISO_FILE	  0	/**< Not really a flag...		  */
-#define	ISO_EXISTENCE	  1	/**< Do not make existence known (hidden) */
-#define	ISO_DIRECTORY	  2	/**< This file is a directory		  */
-#define	ISO_ASSOCIATED	  4	/**< This file is an associated file	  */
-#define	ISO_RECORD	  8	/**< Record format in extended attr. != 0 */
-#define	ISO_PROTECTION	 16	/**< No read/execute perm. in ext. attr.  */
-#define	ISO_DRESERVED1	 32	/**< Reserved bit 5			  */
-#define	ISO_DRESERVED2	 64	/**< Reserved bit 6			  */
-#define	ISO_MULTIEXTENT	128	/**< Not final entry of a mult. ext. file */
-
-/**! Volume descriptor types */
-#define ISO_VD_BOOT_RECORD         0  /**< CD is bootable */
-#define ISO_VD_PRIMARY             1  /**< Is in any ISO-9660 */
-#define ISO_VD_SUPPLEMENTARY	   2  /**< Used by Joliet, for example */
-#define ISO_VD_PARITION	           3  /**< Indicates a partition of a CD */
-#define ISO_VD_END	         255
-
-/*! Sector of Primary Volume Descriptor */
-#define ISO_PVD_SECTOR  16
-
-/*! Sector of End Volume Descriptor */
-#define ISO_EVD_SECTOR  17  
-
 /*! String inside frame which identifies an ISO 9660 filesystem. This
     string is the "id" field of an iso9660_pvd_t or an iso9660_svd_t.
 */
@@ -194,10 +161,6 @@ extern const char ISO_STANDARD_ID[sizeof("CD001")-1];
 
 #define ISO_STANDARD_ID      "CD001" 
 
-
-/*! Number of bytes in an ISO 9660 block */
-#define ISO_BLOCKSIZE           2048 
-
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -264,11 +227,6 @@ typedef struct iso9660_ltime_s  iso9660_ltime_t;
 typedef struct iso9660_dir_s    iso9660_dir_t;
 typedef struct iso9660_stat_s   iso9660_stat_t;
 
-typedef uint32_t posix_mode_t;
-typedef uint32_t posix_nlink_t;
-typedef uint32_t posix_uid_t;
-typedef uint32_t posix_gid_t;
-
 #include <cdio/rock.h>
 
 /*! \brief Format of an ISO-9660 directory record 
@@ -542,6 +500,7 @@ struct iso9660_stat_s { /* big endian!! */
   uint32_t           secsize;         /**< number of sectors allocated */
   iso9660_xa_t       xa;              /**< XA attributes */
   enum { _STAT_FILE = 1, _STAT_DIR = 2 } type;
+  bool               b_xa;
   char         filename[EMPTY_ARRAY_SIZE]; /**< filename */
 };
 
@@ -549,7 +508,7 @@ struct iso9660_stat_s { /* big endian!! */
     of extensions we allow, eg. Joliet, Rock Ridge, etc. */
 typedef uint8_t iso_extension_mask_t;
 
-/*! An enumeration for some of the ISO_EXTENSION_* #defines below. This isn't
+/*! An enumeration for some of the ISO_EXTENSION_* \#defines below. This isn't
     really an enumeration one would really use in a program it is here
     to be helpful in debuggers where wants just to refer to the
     ISO_EXTENSION_*_ names and get something.
@@ -563,12 +522,6 @@ extern enum iso_extension_enum_s {
 } iso_extension_enums;
   
 
-#define ISO_EXTENSION_JOLIET_LEVEL1 0x01
-#define ISO_EXTENSION_JOLIET_LEVEL2 0x02
-#define ISO_EXTENSION_JOLIET_LEVEL3 0x04
-#define ISO_EXTENSION_ROCK_RIDGE    0x08
-#define ISO_EXTENSION_HIGH_SIERRA   0x10
-
 #define ISO_EXTENSION_ALL           0xFF
 #define ISO_EXTENSION_NONE          0x00
 #define ISO_EXTENSION_JOLIET     \
@@ -580,84 +533,93 @@ extern enum iso_extension_enum_s {
 /** This is an opaque structure. */
 typedef struct _iso9660_s iso9660_t; 
 
-/*!
-  Open an ISO 9660 image for reading. Maybe in the future we will have
-  a mode. NULL is returned on error.
-*/
-  iso9660_t *iso9660_open (const char *psz_path /*flags, mode */);
+  /*! Close previously opened ISO 9660 image and free resources
+    associated with the image. Call this when done using using an ISO
+    9660 image.
+    
+    @return true is unconditionally returned. If there was an error
+    false would be returned.
+  */
+  bool iso9660_close (iso9660_t * p_iso);
 
-/*!
-  Open an ISO 9660 image for reading allowing various ISO 9660
-  extensions.  Maybe in the future we will have a mode. NULL is
-  returned on error.
-*/
+
+  /*!
+    Open an ISO 9660 image for reading. Maybe in the future we will have
+    a mode. NULL is returned on error.
+  */
+  iso9660_t *iso9660_open (const char *psz_path /*flags, mode */);
+  
+  /*!
+    Open an ISO 9660 image for reading allowing various ISO 9660
+    extensions.  Maybe in the future we will have a mode. NULL is
+    returned on error.
+
+    @see iso9660_open_fuzzy
+  */
   iso9660_t *iso9660_open_ext (const char *psz_path, 
                                iso_extension_mask_t iso_extension_mask);
+  
+  /*! Open an ISO 9660 image for "fuzzy" reading. This means that we
+    will try to guess various internal offset based on internal
+    checks. This may be useful when trying to read an ISO 9660 image
+    contained in a file format that libiso9660 doesn't know natively
+    (or knows imperfectly.)
 
-/*!
-  Open an ISO 9660 image for reading with some tolerence for positioning
-  of the ISO9660 image. Here we will scan for ISO_STANDARD_ID and set
-  that position as the 
-  Maybe in the future we will have
-  a mode. NULL is returned on error.
+    Some tolerence allowed for positioning the ISO 9660 image. We scan
+    for STANDARD_ID and use that to set the eventual offset to adjust
+    by (as long as that is <= i_fuzz).
 
-  @see iso9660_open
-*/
+    Maybe in the future we will have a mode. NULL is returned on error.
+    
+    @see iso9660_open, @see iso9660_fuzzy_ext
+  */
   iso9660_t *iso9660_open_fuzzy (const char *psz_path /*flags, mode */,
                                  uint16_t i_fuzz);
 
-/*!
-  Open an ISO 9660 image for reading with some tolerence for positioning
-  of the ISO9660 image. Here we will scan for ISO_STANDARD_ID and set
-  that position as the 
-  Maybe in the future we will have
-  a mode. NULL is returned on error.
-
-  @see iso9660_open
-*/
+  /*!
+    Open an ISO 9660 image for reading with some tolerence for positioning
+    of the ISO9660 image. We scan for ISO_STANDARD_ID and use that to set
+    the eventual offset to adjust by (as long as that is <= i_fuzz).
+    
+    Maybe in the future we will have a mode. NULL is returned on error.
+    
+    @see iso9660_open_ext @see iso9660_open_fuzzy
+  */
   iso9660_t *iso9660_open_fuzzy_ext (const char *psz_path,
                                      iso_extension_mask_t iso_extension_mask,
                                      uint16_t i_fuzz
                                      /*flags, mode */);
 
-/*!
-  Read the Super block of an ISO 9660 image but determine framesize
-  and datastart and a possible additional offset. Generally here we are
-  not reading an ISO 9660 image but a CD-Image which contains an ISO 9660
-  filesystem.
-*/
+  /*!
+    Read the Super block of an ISO 9660 image but determine framesize
+    and datastart and a possible additional offset. Generally here we are
+    not reading an ISO 9660 image but a CD-Image which contains an ISO 9660
+    filesystem.
+  */
   bool iso9660_ifs_fuzzy_read_superblock (iso9660_t *p_iso, 
                                           iso_extension_mask_t iso_extension_mask,
                                           uint16_t i_fuzz);
   
-/*!
-  Close previously opened ISO 9660 image.
-  True is unconditionally returned. If there was an error false would
-  be returned.
-*/
-  bool iso9660_close (iso9660_t * p_iso);
-
-
-/*!
-  Seek to a position and then read n bytes. Size read is returned.
-*/
+  /*!
+    Seek to a position and then read n bytes. Size read is returned.
+  */
   long int iso9660_iso_seek_read (const iso9660_t *p_iso, void *ptr, 
                                   lsn_t start, long int i_size);
-
-/*!
-  Read the Primary Volume Descriptor for a CD.
-  True is returned if read, and false if there was an error.
-*/
+  
+  /*!
+    Read the Primary Volume Descriptor for a CD.
+    True is returned if read, and false if there was an error.
+  */
   bool iso9660_fs_read_pvd ( const CdIo_t *p_cdio, 
                              /*out*/ iso9660_pvd_t *p_pvd );
-
-/*!
-  Read the Primary Volume Descriptor for an ISO 9660 image.
-  True is returned if read, and false if there was an error.
-*/
+  
+  /*!
+    Read the Primary Volume Descriptor for an ISO 9660 image.
+    True is returned if read, and false if there was an error.
+  */
   bool iso9660_ifs_read_pvd (const iso9660_t *p_iso, 
                              /*out*/ iso9660_pvd_t *p_pvd);
-
+  
 /*!
   Read the Super block of an ISO 9660 image. This is the 
   Primary Volume Descriptor (PVD) and perhaps a Supplemental Volume 
@@ -665,12 +627,12 @@ typedef struct _iso9660_s iso9660_t;
 */
   bool iso9660_fs_read_superblock (CdIo_t *p_cdio, 
                                    iso_extension_mask_t iso_extension_mask);
-
-/*!
-  Read the Supper block of an ISO 9660 image. This is the 
-  Primary Volume Descriptor (PVD) and perhaps a Supplemental Volume 
-  Descriptor if (Joliet) extensions are acceptable.
-*/
+  
+  /*!
+    Read the Super block of an ISO 9660 image. This is the 
+    Primary Volume Descriptor (PVD) and perhaps a Supplemental Volume 
+    Descriptor if (Joliet) extensions are acceptable.
+  */
   bool iso9660_ifs_read_superblock (iso9660_t *p_iso,
                                     iso_extension_mask_t iso_extension_mask);
 
@@ -685,119 +647,130 @@ typedef struct _iso9660_s iso9660_t;
                           /*out*/ iso9660_dtime_t *idr_date);
 
 
-/*!
-  Set "long" time in format used in ISO 9660 primary volume descriptor
-  from a Unix time structure. */
+  /*!
+    Set "long" time in format used in ISO 9660 primary volume descriptor
+    from a Unix time structure. */
   void iso9660_set_ltime (const struct tm *_tm, 
                           /*out*/ iso9660_ltime_t *p_pvd_date);
-
-/*!
-  Get Unix time structure from format use in an ISO 9660 directory index 
-  record. Even though tm_wday and tm_yday fields are not explicitly in
-  idr_date, they are calculated from the other fields.
-
-  If tm is to reflect the localtime, set "use_localtime" true, otherwise
-  tm will reported in GMT.
-*/
+  
+  /*!
+    Get Unix time structure from format use in an ISO 9660 directory index 
+    record. Even though tm_wday and tm_yday fields are not explicitly in
+    idr_date, they are calculated from the other fields.
+    
+    If tm is to reflect the localtime, set "use_localtime" true, otherwise
+    tm will reported in GMT.
+  */
   bool iso9660_get_dtime (const iso9660_dtime_t *idr_date, bool use_localtime,
                           /*out*/ struct tm *tm);
-
-
-/*!
-  Get "long" time in format used in ISO 9660 primary volume descriptor
-  from a Unix time structure. 
-*/
+  
+  
+  /*!
+    Get "long" time in format used in ISO 9660 primary volume descriptor
+    from a Unix time structure. 
+  */
   bool iso9660_get_ltime (const iso9660_ltime_t *p_ldate, 
                           /*out*/ struct tm *p_tm);
-
-/*====================================================
-  Characters used in file and directory and manipulation
- ====================================================*/
-/*!
-   Return true if c is a DCHAR - a character that can appear in an an
-   ISO-9600 level 1 directory name. These are the ASCII capital
-   letters A-Z, the digits 0-9 and an underscore.
-*/
-bool iso9660_isdchar (int c);
-
-/*!
-   Return true if c is an ACHAR - 
-   These are the DCHAR's plus some ASCII symbols including the space 
-   symbol.   
-*/
-bool iso9660_isachar (int c);
-
-/*!
-   Convert ISO-9660 file name that stored in a directory entry into 
-   what's usually listed as the file name in a listing.
-   Lowercase name, and remove trailing ;1's or .;1's and
-   turn the other ;'s into version numbers.
-
-   The length of the translated string is returned.
-*/
-int iso9660_name_translate(const char *psz_oldname, char *psz_newname);
-
-/*!
-   Convert ISO-9660 file name that stored in a directory entry into
-   what's usually listed as the file name in a listing.  Lowercase
-   name if not using Joliet extension. Remove trailing ;1's or .;1's and
-   turn the other ;'s into version numbers.
-
-   The length of the translated string is returned.
-*/
-int iso9660_name_translate_ext(const char *psz_old, char *psz_new, 
-                               uint8_t i_joliet_level);
   
-/*!  
-  Pad string src with spaces to size len and copy this to dst. If
-  len is less than the length of src, dst will be truncated to the
-  first len characters of src.
-
-  src can also be scanned to see if it contains only ACHARs, DCHARs, 
-  7-bit ASCII chars depending on the enumeration _check.
-
-  In addition to getting changed, dst is the return value.
-  Note: this string might not be NULL terminated.
- */
-char *iso9660_strncpy_pad(char dst[], const char src[], size_t len, 
-                          enum strncpy_pad_check _check);
-
-/*=====================================================================
-  file/dirname's 
-======================================================================*/
-
-/*!
-  Check that psz_path is a valid ISO-9660 directory name.
-
-  A valid directory name should not start out with a slash (/), 
-  dot (.) or null byte, should be less than 37 characters long, 
-  have no more than 8 characters in a directory component 
-  which is separated by a /, and consist of only DCHARs. 
-
-  True is returned if psz_path is valid.
- */
-bool iso9660_dirname_valid_p (const char psz_path[]);
-
-/*!  
-  Take psz_path and a version number and turn that into a ISO-9660
-  pathname.  (That's just the pathname followd by ";" and the version
-  number. For example, mydir/file.ext -> MYDIR/FILE.EXT;1 for version
-  1. The resulting ISO-9660 pathname is returned.
-*/
-char *iso9660_pathname_isofy (const char psz_path[], uint16_t i_version);
-
-/*!
-  Check that psz_path is a valid ISO-9660 pathname.  
-
-  A valid pathname contains a valid directory name, if one appears and
-  the filename portion should be no more than 8 characters for the
-  file prefix and 3 characters in the extension (or portion after a
-  dot). There should be exactly one dot somewhere in the filename
-  portion and the filename should be composed of only DCHARs.
+  /*====================================================
+    Character Classification and String Manipulation
+    ====================================================*/
+  /*!
+    Return true if c is a DCHAR - a character that can appear in an an
+    ISO-9600 level 1 directory name. These are the ASCII capital
+    letters A-Z, the digits 0-9 and an underscore.
+  */
+  bool iso9660_is_dchar (int c);
   
-  True is returned if psz_path is valid.
- */
-bool iso9660_pathname_valid_p (const char psz_path[]);
+  /*!
+    Return true if c is an ACHAR - 
+    These are the DCHAR's plus some ASCII symbols including the space 
+    symbol.   
+  */
+  bool iso9660_is_achar (int c);
+  
+  /*!
+    Convert an ISO-9660 file name which is in the format usually stored
+    in a ISO 9660 directory entry into what's usually listed as the
+    file name in a listing.  Lowercase name, and remove trailing ;1's
+    or .;1's and turn the other ;'s into version numbers.
+    
+    @param psz_oldname the ISO-9660 filename to be translated.
+    @param psz_newname returned string. The caller allocates this and
+    it should be at least the size of psz_oldname.
+    @return length of the translated string is returned.
+  */
+  int iso9660_name_translate(const char *psz_oldname, 
+                             /*out*/ char *psz_newname);
+  
+  /*!
+    Convert an ISO-9660 file name which is in the format usually stored
+    in a ISO 9660 directory entry into what's usually listed as the
+    file name in a listing.  Lowercase name if no Joliet Extension
+    interpretation. Remove trailing ;1's or .;1's and turn the other
+    ;'s into version numbers.
+    
+    @param psz_oldname the ISO-9660 filename to be translated.
+    @param psz_newname returned string. The caller allocates this and
+    it should be at least the size of psz_oldname.
+    @param i_joliet_level 0 if not using Joliet Extension. Otherwise the
+    Joliet level.
+    @return length of the translated string is returned. It will be no greater
+    than the length of psz_oldname.
+  */
+  int iso9660_name_translate_ext(const char *psz_oldname, char *psz_newname, 
+                                 uint8_t i_joliet_level);
+  
+  /*!  
+    Pad string src with spaces to size len and copy this to dst. If
+    len is less than the length of src, dst will be truncated to the
+    first len characters of src.
+    
+    src can also be scanned to see if it contains only ACHARs, DCHARs, 
+    7-bit ASCII chars depending on the enumeration _check.
+    
+    In addition to getting changed, dst is the return value.
+    Note: this string might not be NULL terminated.
+  */
+  char *iso9660_strncpy_pad(char dst[], const char src[], size_t len, 
+                            enum strncpy_pad_check _check);
+  
+  /*=====================================================================
+    File and Directory Names 
+    ======================================================================*/
+  
+  /*!
+    Check that psz_path is a valid ISO-9660 directory name.
+    
+    A valid directory name should not start out with a slash (/), 
+    dot (.) or null byte, should be less than 37 characters long, 
+    have no more than 8 characters in a directory component 
+    which is separated by a /, and consist of only DCHARs. 
+    
+    True is returned if psz_path is valid.
+  */
+  bool iso9660_dirname_valid_p (const char psz_path[]);
+  
+  /*!  
+    Take psz_path and a version number and turn that into a ISO-9660
+    pathname.  (That's just the pathname followd by ";" and the version
+    number. For example, mydir/file.ext -> MYDIR/FILE.EXT;1 for version
+    1. The resulting ISO-9660 pathname is returned.
+  */
+  char *iso9660_pathname_isofy (const char psz_path[], uint16_t i_version);
+  
+  /*!
+    Check that psz_path is a valid ISO-9660 pathname.  
+    
+    A valid pathname contains a valid directory name, if one appears and
+    the filename portion should be no more than 8 characters for the
+    file prefix and 3 characters in the extension (or portion after a
+    dot). There should be exactly one dot somewhere in the filename
+    portion and the filename should be composed of only DCHARs.
+    
+    True is returned if psz_path is valid.
+  */
+  bool iso9660_pathname_valid_p (const char psz_path[]);
 
 /*=====================================================================
   directory tree 
@@ -828,18 +801,21 @@ iso9660_dir_calc_record_size (unsigned int namelen, unsigned int su_len);
    Given a directory pointer, find the filesystem entry that contains
    lsn and return information about it.
 
-   Returns stat_t of entry if we found lsn, or NULL otherwise.
+   @return stat_t of entry if we found lsn, or NULL otherwise.
+   Caller must free return value.
  */
-iso9660_stat_t *iso9660_find_fs_lsn(CdIo_t *p_cdio, lsn_t i_lsn);
+#define iso9660_fs_find_lsn  iso9660_find_fs_lsn 
+iso9660_stat_t *iso9660_fs_find_lsn(CdIo_t *p_cdio, lsn_t i_lsn);
 
 
 /*!
    Given a directory pointer, find the filesystem entry that contains
    lsn and return information about it.
 
-   Returns stat_t of entry if we found lsn, or NULL otherwise.
+   @return stat_t of entry if we found lsn, or NULL otherwise.
+   Caller must free return value.
  */
-iso9660_stat_t *iso9660_find_ifs_lsn(const iso9660_t *p_iso, lsn_t i_lsn);
+iso9660_stat_t *iso9660_ifs_find_lsn(iso9660_t *p_iso, lsn_t i_lsn);
 
 
 /*!
@@ -852,6 +828,8 @@ iso9660_stat_t *iso9660_fs_stat (CdIo_t *p_cdio, const char psz_path[]);
   Return file status for path name psz_path. NULL is returned on error.
   pathname version numbers in the ISO 9660 name are dropped, i.e. ;1
   is removed and if level 1 ISO-9660 names are lowercased.
+
+  b_mode2 is historical. It is not used.
  */
 iso9660_stat_t *iso9660_fs_stat_translate (CdIo_t *p_cdio, 
                                            const char psz_path[], 
@@ -873,6 +851,8 @@ iso9660_stat_t *iso9660_ifs_stat_translate (iso9660_t *p_iso,
 /*!  Read psz_path (a directory) and return a list of iso9660_stat_t
   pointers for the files inside that directory. The caller must free the
   returned result.
+
+  b_mode2 is historical. It is not used.
 */
 CdioList_t * iso9660_fs_readdir (CdIo_t *p_cdio, const char psz_path[], 
                                  bool b_mode2);
@@ -909,131 +889,141 @@ uint8_t iso9660_get_dir_size(const iso9660_dir_t *p_idr);
 lsn_t iso9660_get_dir_extent(const iso9660_dir_t *p_idr);
 #endif
 
-/*!
-  Return the directory name stored in the iso9660_dir_t
-
-  A string is allocated: the caller must deallocate.
-*/
-char * iso9660_dir_to_name (const iso9660_dir_t *p_iso9660_dir);
+  /*!
+    Return the directory name stored in the iso9660_dir_t
+    
+    A string is allocated: the caller must deallocate.
+  */
+  char * iso9660_dir_to_name (const iso9660_dir_t *p_iso9660_dir);
   
-/*!
-   Return a string containing the preparer id with trailing
-   blanks removed.
-*/
-char *iso9660_get_preparer_id(const iso9660_pvd_t *p_pvd);
+  /*!  
+    Returns a POSIX mode for a given p_iso_dirent.
+  */
+  mode_t iso9660_get_posix_filemode(const iso9660_stat_t *p_iso_dirent);
+
+  /*!
+    Return a string containing the preparer id with trailing
+    blanks removed.
+  */
+  char *iso9660_get_preparer_id(const iso9660_pvd_t *p_pvd);
   
-/*!  
-  Get the preparer ID.  psz_preparer_id is set to NULL if there
-  is some problem in getting this and false is returned.
-*/
-bool iso9660_ifs_get_preparer_id(iso9660_t *p_iso,
-                                 /*out*/ char **p_psz_preparer_id);
+  /*!  
+    Get the preparer ID.  psz_preparer_id is set to NULL if there
+    is some problem in getting this and false is returned.
+  */
+  bool iso9660_ifs_get_preparer_id(iso9660_t *p_iso,
+                                   /*out*/ char **p_psz_preparer_id);
   
-/*!
-   Return a string containing the PVD's publisher id with trailing
-   blanks removed.
-*/
-char *iso9660_get_publisher_id(const iso9660_pvd_t *p_pvd);
-
-/*!  
-  Get the publisher ID.  psz_publisher_id is set to NULL if there
-  is some problem in getting this and false is returned.
-*/
-bool iso9660_ifs_get_publisher_id(iso9660_t *p_iso,
-                                  /*out*/ char **p_psz_publisher_id);
-
-uint8_t iso9660_get_pvd_type(const iso9660_pvd_t *p_pvd);
-
-const char * iso9660_get_pvd_id(const iso9660_pvd_t *p_pvd);
-
-int iso9660_get_pvd_space_size(const iso9660_pvd_t *p_pvd);
-
-int iso9660_get_pvd_block_size(const iso9660_pvd_t *p_pvd) ;
-
-/*! Return the primary volume id version number (of pvd).
+  /*!
+    Return a string containing the PVD's publisher id with trailing
+    blanks removed.
+  */
+  char *iso9660_get_publisher_id(const iso9660_pvd_t *p_pvd);
+  
+  /*!  
+    Get the publisher ID.  psz_publisher_id is set to NULL if there
+    is some problem in getting this and false is returned.
+  */
+  bool iso9660_ifs_get_publisher_id(iso9660_t *p_iso,
+                                    /*out*/ char **p_psz_publisher_id);
+  
+  uint8_t iso9660_get_pvd_type(const iso9660_pvd_t *p_pvd);
+  
+  const char * iso9660_get_pvd_id(const iso9660_pvd_t *p_pvd);
+  
+  int iso9660_get_pvd_space_size(const iso9660_pvd_t *p_pvd);
+  
+  int iso9660_get_pvd_block_size(const iso9660_pvd_t *p_pvd) ;
+  
+  /*! Return the primary volume id version number (of pvd).
     If there is an error 0 is returned. 
- */
-int iso9660_get_pvd_version(const iso9660_pvd_t *pvd) ;
-
-/*!
-   Return a string containing the PVD's system id with trailing
-   blanks removed.
-*/
-char *iso9660_get_system_id(const iso9660_pvd_t *p_pvd);
+  */
+  int iso9660_get_pvd_version(const iso9660_pvd_t *pvd) ;
   
-/*!  
-  Get the system ID.  psz_system_id is set to NULL if there
-  is some problem in getting this and false is returned.
-*/
-bool iso9660_ifs_get_system_id(iso9660_t *p_iso,
-                               /*out*/ char **p_psz_system_id);
-
-
-/*! Return the LSN of the root directory for pvd.
+  /*!
+    Return a string containing the PVD's system id with trailing
+    blanks removed.
+  */
+  char *iso9660_get_system_id(const iso9660_pvd_t *p_pvd);
+  
+  /*!  
+    Get the system ID.  psz_system_id is set to NULL if there
+    is some problem in getting this and false is returned.
+  */
+  bool iso9660_ifs_get_system_id(iso9660_t *p_iso,
+                                 /*out*/ char **p_psz_system_id);
+  
+  
+  /*! Return the LSN of the root directory for pvd.
     If there is an error CDIO_INVALID_LSN is returned. 
- */
-lsn_t iso9660_get_root_lsn(const iso9660_pvd_t *p_pvd);
-
-/*!
-  Return the PVD's volume ID.
-*/
-char *iso9660_get_volume_id(const iso9660_pvd_t *p_pvd);
-
-/*!  
-  Get the system ID.  psz_system_id is set to NULL if there
-  is some problem in getting this and false is returned.
-*/
-bool iso9660_ifs_get_volume_id(iso9660_t *p_iso,
-                               /*out*/ char **p_psz_volume_id);
-
-/*!
-  Return the PVD's volumeset ID.
-  NULL is returned if there is some problem in getting this. 
-*/
-char *iso9660_get_volumeset_id(const iso9660_pvd_t *p_pvd);
-
-/*!  
-  Get the systemset ID.  psz_systemset_id is set to NULL if there
-  is some problem in getting this and false is returned.
-*/
-bool iso9660_ifs_get_volumeset_id(iso9660_t *p_iso,
-                                  /*out*/ char **p_psz_volumeset_id);
-
-/* pathtable */
-
-/*! Zero's out pathable. Do this first. */
-void iso9660_pathtable_init (void *pt);
-
-unsigned int iso9660_pathtable_get_size (const void *pt);
-
-uint16_t
-iso9660_pathtable_l_add_entry (void *pt, const char name[], uint32_t extent,
-                               uint16_t parent);
-
-uint16_t
-iso9660_pathtable_m_add_entry (void *pt, const char name[], uint32_t extent,
-                               uint16_t parent);
-
+  */
+  lsn_t iso9660_get_root_lsn(const iso9660_pvd_t *p_pvd);
+  
+  /*!  
+    Get the volume ID in the PVD.  psz_volume_id is set to NULL if there
+    is some problem in getting this and false is returned.
+  */
+  char *iso9660_get_volume_id(const iso9660_pvd_t *p_pvd);
+  
+  /*!  
+    Get the volume ID in the PVD.  psz_volume_id is set to NULL if there
+    is some problem in getting this and false is returned.
+  */
+  bool iso9660_ifs_get_volume_id(iso9660_t *p_iso,
+                                 /*out*/ char **p_psz_volume_id);
+  
+  /*!
+    Return the volumeset ID in the PVD.
+    NULL is returned if there is some problem in getting this. 
+  */
+  char *iso9660_get_volumeset_id(const iso9660_pvd_t *p_pvd);
+  
+  /*!  
+    Get the volumeset ID.  psz_systemset_id is set to NULL if there
+    is some problem in getting this and false is returned.
+  */
+  bool iso9660_ifs_get_volumeset_id(iso9660_t *p_iso,
+                                    /*out*/ char **p_psz_volumeset_id);
+  
+  /* pathtable */
+  
+  /*! Zero's out pathable. Do this first. */
+  void iso9660_pathtable_init (void *pt);
+  
+  unsigned int iso9660_pathtable_get_size (const void *pt);
+  
+  uint16_t iso9660_pathtable_l_add_entry (void *pt, const char name[], 
+                                          uint32_t extent, uint16_t parent);
+  
+  uint16_t iso9660_pathtable_m_add_entry (void *pt, const char name[], 
+                                          uint32_t extent, uint16_t parent);
+  
   /**=====================================================================
      Volume Descriptors
      ======================================================================*/
 
-void
-iso9660_set_pvd (void *pd, const char volume_id[], const char application_id[],
-                 const char publisher_id[], const char preparer_id[],
-                 uint32_t iso_size, const void *root_dir, 
-                 uint32_t path_table_l_extent, uint32_t path_table_m_extent,
-                 uint32_t path_table_size, const time_t *pvd_time);
+  void iso9660_set_pvd (void *pd, const char volume_id[], 
+                        const char application_id[], 
+                        const char publisher_id[], const char preparer_id[],
+                        uint32_t iso_size, const void *root_dir, 
+                        uint32_t path_table_l_extent, 
+                        uint32_t path_table_m_extent,
+                        uint32_t path_table_size, const time_t *pvd_time);
 
-void 
-iso9660_set_evd (void *pd);
+  void iso9660_set_evd (void *pd);
 
-/*!
-  Return true if ISO 9660 image has extended attrributes (XA).
-*/
-bool iso9660_ifs_is_xa (const iso9660_t * p_iso);
+  /*!
+    Return true if ISO 9660 image has extended attrributes (XA).
+  */
+  bool iso9660_ifs_is_xa (const iso9660_t * p_iso);
 
 
+#ifndef DO_NOT_WANT_COMPATIBILITY
+/** For compatibility with < 0.77 */
+#define iso9660_isdchar       iso9660_is_dchar
+#define iso9660_isachar       iso9660_is_achar
+#endif /*DO_NOT_WANT_COMPATIBILITY*/
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
