@@ -1,8 +1,9 @@
 /*
-    $Id: types.h,v 1.32 2005/09/17 20:57:19 rocky Exp $
+    $Id: types.h,v 1.35 2006/01/23 20:47:33 rocky Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
-    Copyright (C) 2002, 2003, 2004, 2005 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2002, 2003, 2004, 2005, 2006
+    Rocky Bernstein <rocky@panix.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,8 +32,13 @@
 extern "C" {
 #endif /* __cplusplus */
 
-#ifndef LIBCDIO_CONFIG_H
+#ifndef EXTERNAL_LIBCDIO_CONFIG_H
+#define EXTERNAL_LIBCDIO_CONFIG_H
 #include <cdio/cdio_config.h>
+#endif
+
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
 #endif
 
   /* provide some C99 definitions */
@@ -55,6 +61,8 @@ extern "C" {
   /* fixme */
 #endif /* HAVE_STDINT_H */
   
+typedef uint8_t ubyte;
+
   /* default HP/UX macros are broken */
 #if defined(__hpux__)
 # undef UINT16_C
