@@ -1,8 +1,8 @@
 /*
-    $Id: util.h,v 1.8 2005/04/27 23:31:47 rocky Exp $
+    $Id: util.h,v 1.10 2006/03/18 00:53:20 rocky Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
-    Copyright (C) 2004, 2005 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2004, 2005, 2006 Rocky Bernstein <rocky@panix.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -91,9 +91,6 @@ _cdio_strdup_upper (const char str[]);
 void
 _cdio_strfreev(char **strv);
 
-char *
-_cdio_strjoin (char *strv[], unsigned count, const char delim[]);
-
 size_t
 _cdio_strlenv(char **str_array);
 
@@ -103,20 +100,6 @@ _cdio_strsplit(const char str[], char delim);
 uint8_t cdio_to_bcd8(uint8_t n);
 uint8_t cdio_from_bcd8(uint8_t p);
 
-#if defined(__GNUC__) && __GNUC__ >= 3
-static inline __attribute__((deprecated))
-uint8_t to_bcd8(uint8_t n) {
-  return cdio_to_bcd8(n);
-}
-static inline __attribute__((deprecated))
-uint8_t from_bcd8(uint8_t p) {
-  return cdio_from_bcd8(p);
-}
-#else
-#define to_bcd8 cdio_to_bcd8
-#define from_bcd8 cdio_from_bcd8
-#endif
-
 #ifdef __cplusplus
 }
 #endif
