@@ -1,7 +1,8 @@
 /*
-    $Id: cd_types.c,v 1.3 2005/02/17 11:54:28 rocky Exp $
+    $Id: cd_types.c,v 1.6 2006/02/27 10:29:20 flameeyes Exp $
 
-    Copyright (C) 2003, 2004, 2005 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2003, 2004, 2005, 2006 Rocky Bernstein
+    <rocky@panix.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -60,6 +61,14 @@ and
 
 */
 
+/** The below variables are trickery to force enum symbol values to be
+    recorded in debug symbol tables. They are used to allow one to refer
+    to the enumeration value names in the typedefs above in a debugger
+    and debugger expressions.
+*/
+cdio_fs_cap_t debug_cdio_fs_cap;
+cdio_fs_t     debug_cdio_fs;
+
 static char buffer[6][CDIO_CD_FRAMESIZE_RAW];  /* for CD-Data */
 
 /* Some interesting sector numbers stored in the above buffer. */
@@ -98,7 +107,7 @@ static signature_t sigs[] =
     {3,     7, "EL TORITO",  "BOOTABLE"}, 
     {4,     0, "VIDEO_CD",   "VIDEO CD"}, 
     {4,     0, "SUPERVCD",   "SVCD or Chaoji VCD"}, 
-    { 0 }
+    {0, 0, NULL, NULL }
   };
 
 
