@@ -1,7 +1,7 @@
 /*
-    $Id: device.c,v 1.27 2005/04/11 01:03:46 rocky Exp $
+    $Id: device.c,v 1.35 2006/02/27 10:29:20 flameeyes Exp $
 
-    Copyright (C) 2005 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2005, 2006 Rocky Bernstein <rocky@panix.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -48,7 +48,7 @@
 /* The below array gives of the drivers that are currently available for 
    on a particular host. */
 
-CdIo_driver_t CdIo_driver[CDIO_MAX_DRIVER] = { {0} };
+CdIo_driver_t CdIo_driver[CDIO_MAX_DRIVER] = { {0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL} };
 
 /* The last valid entry of Cdio_driver. 
    -1 or (CDIO_DRIVER_UNINIT) means uninitialzed. 
@@ -76,6 +76,17 @@ const driver_id_t cdio_os_driver = DRIVER_WIN32;
 const driver_id_t cdio_os_driver = DRIVER_UNKNOWN;
 #endif
 
+/** The below variables are trickery to force enum symbol values to be
+    recorded in debug symbol tables. They are used to allow one to refer
+    to the enumeration value names in the typedefs above in a debugger
+    and debugger expressions.
+*/
+cdio_drive_cap_misc_t          debug_cdio_drive_cap_misc;
+cdio_drive_cap_read_t          debug_cdio_drive_cap_read_t;
+cdio_drive_cap_write_t         debug_drive_cap_write_t;
+cdio_mmc_hw_len_t              debug_cdio_mmc_hw_len;
+cdio_src_category_mask_t       debug_cdio_src_category_mask;
+
 static bool 
 cdio_have_false(void)
 {
@@ -231,6 +242,31 @@ CdIo_driver_t CdIo_all_drivers[CDIO_MAX_DRIVER+1] = {
 
 };
 
+const char *
+cdio_driver_errmsg(driver_return_code_t drc)
+{
+  switch(drc) {
+  case DRIVER_OP_SUCCESS: 
+    return "driver operation was successful";
+  case DRIVER_OP_ERROR:
+    return "driver I/O error";
+  case DRIVER_OP_UNSUPPORTED:
+    return "driver operatation not supported";
+  case DRIVER_OP_UNINIT:
+    return "driver not initialized";
+  case DRIVER_OP_NOT_PERMITTED:
+    return "driver operatation not permitted";
+  case DRIVER_OP_BAD_PARAMETER:
+    return "bad parameter passed";
+  case DRIVER_OP_BAD_POINTER:
+    return "bad pointer to memory area";
+  case DRIVER_OP_NO_DRIVER:
+    return "driver not available";
+  default:
+    return "unknown or bad driver return status";
+  }
+}
+
 static CdIo *
 scan_for_driver(driver_id_t start, driver_id_t end, 
                 const char *psz_source, const char *access_mode) 
@@ -302,19 +338,27 @@ cdio_destroy (CdIo_t *p_cdio)
 /*!
   Close media tray in CD drive if there is a routine to do so. 
   
-  @param psz_drive the name of CD-ROM to be closed.
+  @param psz_drive the name of CD-ROM to be closed. If NULL, we will
+  use the default device.
   @param p_driver_id is the driver to be used or that got used if
   it was DRIVER_UNKNOWN or DRIVER_DEVICE; If this is NULL, we won't
   report back the driver used.
 */
 driver_return_code_t 
-cdio_close_tray (const char *psz_drive, /*in/out*/ driver_id_t
+cdio_close_tray (const char *psz_orig_drive, /*in/out*/ driver_id_t
                  *p_driver_id)
 {
   driver_id_t temp_driver_id = DRIVER_DEVICE;
+  char *psz_drive;
+  driver_return_code_t drc;
 
   if (!p_driver_id) p_driver_id = &temp_driver_id;
   
+  if (!psz_orig_drive || !*psz_orig_drive)
+    psz_drive = cdio_get_default_device_driver(p_driver_id);
+  else 
+    psz_drive = strdup(psz_orig_drive);
+  
   if (DRIVER_UNKNOWN == *p_driver_id || DRIVER_DEVICE == *p_driver_id) {
     *p_driver_id = CDIO_MIN_DEVICE_DRIVER;
     
@@ -322,7 +366,9 @@ cdio_close_tray (const char *psz_drive, /*in/out*/ driver_id_t
     for ( ; *p_driver_id<=CDIO_MAX_DRIVER; (*p_driver_id)++) {
       if ( (*CdIo_all_drivers[*p_driver_id].have_driver)() &&
            *CdIo_all_drivers[*p_driver_id].close_tray ) {
-        return (*CdIo_all_drivers[*p_driver_id].close_tray)(psz_drive);
+        drc = (*CdIo_all_drivers[*p_driver_id].close_tray)(psz_drive);
+        free(psz_drive);
+        return drc;
       }
     }
     return DRIVER_OP_UNSUPPORTED;
@@ -331,7 +377,9 @@ cdio_close_tray (const char *psz_drive, /*in/out*/ driver_id_t
   /* The driver id was specified. Use that. */
   if ( (*CdIo_all_drivers[*p_driver_id].have_driver)() &&
        *CdIo_all_drivers[*p_driver_id].close_tray ) {
-    return (*CdIo_all_drivers[*p_driver_id].close_tray)(psz_drive);
+    drc = (*CdIo_all_drivers[*p_driver_id].close_tray)(psz_drive);
+    free(psz_drive);
+    return drc;
   }
   return DRIVER_OP_UNSUPPORTED;
 }
@@ -345,7 +393,6 @@ cdio_close_tray (const char *psz_drive, /*in/out*/ driver_id_t
 driver_return_code_t
 cdio_eject_media (CdIo_t **pp_cdio)
 {
-  
   if ((pp_cdio == NULL) || (*pp_cdio == NULL)) return DRIVER_OP_UNINIT;
 
   if ((*pp_cdio)->op.eject_media) {
@@ -362,6 +409,24 @@ cdio_eject_media (CdIo_t **pp_cdio)
   }
 }
 
+/*!
+  Eject media in CD drive if there is a routine to do so. If you want
+  to scan for any CD-ROM and eject that, pass NULL for psz_drive.
+  
+  @param psz_drive the CD object to be acted upon.
+  If NULL is given as the drive, we'll use the default driver device.
+*/
+driver_return_code_t
+cdio_eject_media_drive (const char *psz_drive)
+{
+  CdIo_t *p_cdio = cdio_open (psz_drive, DRIVER_DEVICE);
+  if (p_cdio) {
+    return cdio_eject_media(&p_cdio);
+  } else {
+    return DRIVER_OP_UNINIT;
+  }
+}
+
 /*!
   Free device list returned by cdio_get_devices or
   cdio_get_devices_with_cap.
@@ -664,7 +729,7 @@ cdio_get_hwinfo (const CdIo_t *p_cdio, cdio_hwinfo_t *hw_info)
     return p_cdio->op.get_hwinfo (p_cdio, hw_info);
   } else {
     /* Perhaps driver forgot to initialize.  We are no worse off Using
-       scsi_mmc than returning false here. */
+      mmc than returning false here. */
     return mmc_get_hwinfo(p_cdio, hw_info);
   }
 }
@@ -778,6 +843,20 @@ cdio_have_driver(driver_id_t driver_id)
 bool
 cdio_is_device(const char *psz_source, driver_id_t driver_id)
 {
+  if (DRIVER_UNKNOWN == driver_id || DRIVER_DEVICE == driver_id) {
+    if (DRIVER_UNKNOWN == driver_id) 
+      driver_id++;
+    else 
+      driver_id = CDIO_MIN_DEVICE_DRIVER;
+    
+    /* Scan for driver */
+    for ( ; driver_id<=CDIO_MAX_DRIVER; driver_id++) {
+      if ( (*CdIo_all_drivers[driver_id].have_driver)() &&
+           CdIo_all_drivers[driver_id].is_device ) {
+        return (*CdIo_all_drivers[driver_id].is_device)(psz_source);
+      }
+    }
+  }
   if (CdIo_all_drivers[driver_id].is_device == NULL) return false;
   return (*CdIo_all_drivers[driver_id].is_device)(psz_source);
 }
@@ -811,7 +890,7 @@ cdio_open_am (const char *psz_orig_source, driver_id_t driver_id,
   
   if (CdIo_last_driver == -1) cdio_init();
 
-  if (NULL == psz_orig_source || strlen(psz_orig_source)==0) 
+  if (!psz_orig_source || !*psz_orig_source)
     psz_source = cdio_get_default_device(NULL);
   else 
     psz_source = strdup(psz_orig_source);
