@@ -1,5 +1,5 @@
 /*
-    $Id: gnu_linux.c,v 1.17 2005/07/23 22:05:29 rocky Exp $
+    $Id: gnu_linux.c,v 1.20 2006/03/18 00:53:20 rocky Exp $
 
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
     Copyright (C) 2002, 2003, 2004, 2005 Rocky Bernstein <rocky@panix.com>
@@ -27,7 +27,7 @@
 # include "config.h"
 #endif
 
-static const char _rcsid[] = "$Id: gnu_linux.c,v 1.17 2005/07/23 22:05:29 rocky Exp $";
+static const char _rcsid[] = "$Id: gnu_linux.c,v 1.20 2006/03/18 00:53:20 rocky Exp $";
 
 #include <string.h>
 
@@ -100,13 +100,14 @@ typedef struct {
 /**** prototypes for static functions ****/
 static bool is_cdrom_linux(const char *drive, char *mnttype);
 static bool read_toc_linux (void *p_user_data);
-static driver_return_code_t run_mmc_cmd_linux( void *p_user_data, 
-                                               unsigned int i_timeout,
-                                               unsigned int i_cdb, 
-                                               const mmc_cdb_t *p_cdb, 
-                                               mmc_direction_t e_direction, 
-                                               unsigned int i_buf, 
-                                               /*in/out*/ void *p_buf );
+static driver_return_code_t 
+run_mmc_cmd_linux( void *p_user_data, 
+                   unsigned int i_timeout,
+                   unsigned int i_cdb, 
+                   const mmc_cdb_t *p_cdb, 
+                   cdio_mmc_direction_t e_direction, 
+                   unsigned int i_buf, 
+                   /*in/out*/ void *p_buf );
 static access_mode_t 
 
 str_to_access_mode_linux(const char *psz_access_mode) 
@@ -139,20 +140,24 @@ check_mounts_linux(const char *mtab)
     char *tmp;
     char *mnt_type;
     char *mnt_dev;
+    unsigned int i_mnt_type;
+    unsigned int i_mnt_dev;
     
     while ( (mntent=getmntent(mntfp)) != NULL ) {
-      mnt_type = malloc(strlen(mntent->mnt_type) + 1);
+      i_mnt_type = strlen(mntent->mnt_type) + 1;
+      mnt_type = calloc(1, i_mnt_type);
       if (mnt_type == NULL)
 	continue;  /* maybe you'll get lucky next time. */
       
-      mnt_dev = malloc(strlen(mntent->mnt_fsname) + 1);
+      i_mnt_dev = strlen(mntent->mnt_fsname) + 1;
+      mnt_dev = calloc(1, i_mnt_dev);
       if (mnt_dev == NULL) {
 	free(mnt_type);
 	continue;
       }
       
-      strcpy(mnt_type, mntent->mnt_type);
-      strcpy(mnt_dev, mntent->mnt_fsname);
+      strncpy(mnt_type, mntent->mnt_type, i_mnt_type);
+      strncpy(mnt_dev, mntent->mnt_fsname, i_mnt_dev);
       
       /* Handle "supermount" filesystem mounts */
       if ( strcmp(mnt_type, "supermount") == 0 ) {
@@ -627,7 +632,9 @@ eject_media_linux (void *p_user_data) {
         }
       }
       /* force kernel to reread partition table when new disc inserted */
-      ret = ioctl(p_env->gen.fd, BLKRRPART);
+      if (0 != ioctl(p_env->gen.fd, BLKRRPART)) {
+        cdio_info ("BLKRRPART request failed: %s\n", strerror(errno));
+      }
       break;
     }
   } else {
@@ -1104,7 +1111,7 @@ static driver_return_code_t
 run_mmc_cmd_linux( void *p_user_data, 
                    unsigned int i_timeout_ms,
                    unsigned int i_cdb, const mmc_cdb_t *p_cdb, 
-                   mmc_direction_t e_direction, 
+                   cdio_mmc_direction_t e_direction, 
                    unsigned int i_buf, /*in/out*/ void *p_buf )
 {
   const _img_private_t *p_env = p_user_data;
