@@ -1,7 +1,7 @@
 /*
-    $Id: osx.c,v 1.3 2005/04/23 01:16:19 rocky Exp $
+    $Id: osx.c,v 1.8 2006/02/18 22:47:41 rocky Exp $
 
-    Copyright (C) 2003, 2004, 2005 Rocky Bernstein <rocky@panix.com> 
+    Copyright (C) 2003, 2004, 2005, 2006 Rocky Bernstein <rocky@panix.com> 
     from vcdimager code: 
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
     and VideoLAN code Copyright (C) 1998-2001 VideoLAN
@@ -34,7 +34,7 @@
 #include "config.h"
 #endif
 
-static const char _rcsid[] = "$Id: osx.c,v 1.3 2005/04/23 01:16:19 rocky Exp $";
+static const char _rcsid[] = "$Id: osx.c,v 1.8 2006/02/18 22:47:41 rocky Exp $";
 
 #include <cdio/logging.h>
 #include <cdio/sector.h>
@@ -92,7 +92,8 @@ static const char _rcsid[] = "$Id: osx.c,v 1.3 2005/04/23 01:16:19 rocky Exp $";
 #define kIOCDBlockStorageDeviceClassString		"IOCDBlockStorageDevice"
 
 /* Note leadout is normally defined 0xAA, But on OSX 0xA0 is "lead in" while
-   0xA2 is "lead out". Don't ask me why. */
+   0xA2 is "lead out". I don't understand the distinction, and therefore
+   something could be wrong. */
 #define	OSX_CDROM_LEADOUT_TRACK 0xA2
 
 #define TOTAL_TRACKS    (p_env->i_last_track - p_env->gen.i_first_track + 1)
@@ -350,7 +351,7 @@ static int
 run_mmc_cmd_osx( void *p_user_data, 
 		 unsigned int i_timeout_ms,
 		 unsigned int i_cdb, const mmc_cdb_t *p_cdb, 
-		 scsi_mmc_direction_t e_direction, 
+		 cdio_mmc_direction_t e_direction, 
 		 unsigned int i_buf, /*in/out*/ void *p_buf )
 {
   _img_private_t *p_env = p_user_data;
@@ -483,7 +484,7 @@ static int
 run_mmc_cmd_osx( const void *p_user_data, 
 		 unsigned int i_timeout_ms,
 		 unsigned int i_cdb, const mmc_cdb_t *p_cdb, 
-		 scsi_mmc_direction_t e_direction, 
+		 cdio_mmc_direction_t e_direction, 
 		 unsigned int i_buf, /*in/out*/ void *p_buf )
 {
 
@@ -1198,12 +1199,12 @@ read_toc_osx (void *p_user_data)
     
     pTrackDescriptors = p_env->pTOC->descriptors;
 
-    p_env->gen.i_first_track   = CDIO_CD_MAX_TRACKS+1;
-    p_env->i_last_track    = CDIO_CD_MIN_TRACK_NO;
-    p_env->i_first_session = CDIO_CD_MAX_TRACKS+1;
-    p_env->i_last_session  = CDIO_CD_MIN_TRACK_NO;
+    p_env->gen.i_first_track = CDIO_CD_MAX_TRACKS+1;
+    p_env->i_last_track      = CDIO_CD_MIN_TRACK_NO;
+    p_env->i_first_session   = CDIO_CD_MAX_TRACKS+1;
+    p_env->i_last_session    = CDIO_CD_MIN_TRACK_NO;
     
-    for( i = 0; i <= p_env->i_descriptors; i++ )
+    for( i = 0; i < p_env->i_descriptors; i++ )
       {
 	track_t i_track     = pTrackDescriptors[i].point;
 	session_t i_session = pTrackDescriptors[i].session;
@@ -1246,7 +1247,7 @@ read_toc_osx (void *p_user_data)
     /* Now that we know what the first track number is, we can make sure
        index positions are ordered starting at 0.
      */
-    for( i = 0; i <= p_env->i_descriptors; i++ )
+    for( i = 0; i < p_env->i_descriptors; i++ )
       {
 	track_t i_track = pTrackDescriptors[i].point;
 
@@ -1627,12 +1628,8 @@ cdio_get_devices_osx(void)
 				  sizeof(psz_buf) - dev_path_length,
 				  kCFStringEncodingASCII ) )
 	    {
-	      CFRelease( str_bsd_path );
-	      IOObjectRelease( next_media );
-	      IOObjectRelease( media_iterator );
 	      cdio_add_device_list(&drives, strdup(psz_buf), &num_drives);
 	    }
-	  
 	  CFRelease( str_bsd_path );
 	  IOObjectRelease( next_media );
 	  
