@@ -1,8 +1,8 @@
 /*
-    $Id: iso9660.c,v 1.14 2005/09/20 00:42:14 rocky Exp $
+    $Id: iso9660.c,v 1.25 2006/03/18 02:35:07 rocky Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
-    Copyright (C) 2003, 2004, 2005 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2003, 2004, 2005, 2006 Rocky Bernstein <rocky@panix.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -43,12 +43,18 @@ const char ISO_STANDARD_ID[] = {'C', 'D', '0', '0', '1'};
 #ifdef HAVE_STDIO_H
 #include <stdio.h>
 #endif
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
 
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
 
-static const char _rcsid[] = "$Id: iso9660.c,v 1.14 2005/09/20 00:42:14 rocky Exp $";
+static const char _rcsid[] = "$Id: iso9660.c,v 1.25 2006/03/18 02:35:07 rocky Exp $";
 
 /* Variables to hold debugger-helping enumerations */
 enum iso_enum1_s     iso_enums1;
@@ -133,32 +139,16 @@ iso9660_get_dtime (const iso9660_dtime_t *idr_date, bool b_localtime,
   p_tm->tm_mday   = idr_date->dt_day;
   p_tm->tm_hour   = idr_date->dt_hour;
   p_tm->tm_min    = idr_date->dt_minute;
-  p_tm->tm_sec    = idr_date->dt_second;
+  p_tm->tm_sec    = idr_date->dt_second - idr_date->dt_gmtoff * (15 * 60);
   p_tm->tm_isdst  = -1; /* information not available */
+
+  /* Recompute tm_wday and tm_yday via mktime. mktime will also renormalize
+     date values to account for the timezone offset. */
   {
     time_t t = 0;
     struct tm *p_temp_tm = b_localtime ? localtime(&t): gmtime(&t);
     
-    /* Recompute tm_wday and tm_yday via mktime. Also adjust for 
-       time-zone differences */
     t = mktime(p_tm);
-    t -= idr_date->dt_gmtoff * (15 * 60);
-#ifdef HAVE_TM_GMTOFF
-    t += p_temp_tm->tm_gmtoff;
-#else 
-    {
-      static bool i_first_time = true;
-      static time_t gmt_off;
-      if (i_first_time) {
-        struct tm tm_local, tm_gmt;
-        memcpy(&tm_local, localtime(&t), sizeof(struct tm));
-        memcpy(&tm_gmt  , gmtime(&t)   , sizeof(struct tm));
-        gmt_off      = mktime(&tm_local) - mktime(&tm_gmt);
-        i_first_time = false;
-      }
-      t += gmt_off;
-    }
-#endif
     
     p_temp_tm = b_localtime ? localtime(&t) : gmtime(&t);
 
@@ -176,7 +166,10 @@ iso9660_get_dtime (const iso9660_dtime_t *idr_date, bool b_localtime,
 
 #define set_ltime_field(TM_FIELD, LT_FIELD, ADD_CONSTANT)               \
   {                                                                     \
-    p_tm->TM_FIELD = strtol(p_ldate->LT_FIELD,                          \
+    char num[10];                                                       \
+    memcpy(num, p_ldate->LT_FIELD, sizeof(p_ldate->LT_FIELD));          \
+    num[sizeof(p_ldate->LT_FIELD)+1] = '\0';                            \
+    p_tm->TM_FIELD = strtol(num,                                        \
                             (char **)NULL, 10)+ADD_CONSTANT;            \
     if (0 != errno) return false;                                       \
   }
@@ -191,24 +184,31 @@ iso9660_get_ltime (const iso9660_ltime_t *p_ldate,
 {
   if (!p_tm) return false;
   memset(p_tm, 0, sizeof(struct tm));
-  set_ltime_field(tm_year, lt_year,   0);
+  set_ltime_field(tm_year, lt_year,  -1900);
   set_ltime_field(tm_mon,  lt_month, -1);
   set_ltime_field(tm_mday, lt_day,    0);
   set_ltime_field(tm_hour, lt_hour,   0);
   set_ltime_field(tm_min,  lt_minute, 0);
   set_ltime_field(tm_sec,  lt_second, 0);
   p_tm->tm_isdst= -1; /* information not available */
-#ifdef HAVE_TM_GMTOFF
-  p_tm->tm_gmtoff = - p_ldate->lt_gmtoff * (15 * 60);
-#endif
+  p_tm->tm_sec += p_ldate->lt_gmtoff * (15 * 60);
 
-  /* Recompute tm_wday and tm_yday via mktime. */
+  /* Recompute tm_wday and tm_yday via mktime. mktime will also renormalize
+     date values to account for the timezone offset. */
   {
     time_t t;
     struct tm *p_temp_tm;
-    
+    const char *old_tzname=getenv("TZ");
+    char psz_gmt_tzset[]="TZ=GMT";
+
+    putenv(psz_gmt_tzset);
+    tzset();
     t = mktime(p_tm);
     p_temp_tm = gmtime(&t);
+    if (old_tzname) {
+      char psz_tzset[10];
+      snprintf(psz_tzset, sizeof(psz_tzset), "TZ=%s", old_tzname);
+    }
     
     p_tm->tm_wday = p_temp_tm->tm_wday;
     p_tm->tm_yday = p_temp_tm->tm_yday;
@@ -239,7 +239,7 @@ iso9660_set_dtime (const struct tm *p_tm, /*out*/ iso9660_dtime_t *p_idr_date)
      represents a 15-minute interval. */
   p_idr_date->dt_gmtoff = p_tm->tm_gmtoff / (15 * 60);
 
-  if (p_tm->tm_isdst) p_idr_date->dt_gmtoff -= 4;
+  if (p_tm->tm_isdst > 0) p_idr_date->dt_gmtoff -= 4;
 
   if (p_idr_date->dt_gmtoff < -48 ) {
     
@@ -260,45 +260,73 @@ iso9660_set_dtime (const struct tm *p_tm, /*out*/ iso9660_dtime_t *p_idr_date)
   Set "long" time in format used in ISO 9660 primary volume descriptor
   from a Unix time structure. */
 void
-iso9660_set_ltime (const struct tm *_tm, /*out*/ iso9660_ltime_t *pvd_date)
+iso9660_set_ltime (const struct tm *p_tm, /*out*/ iso9660_ltime_t *pvd_date)
 {
   char *_pvd_date = (char *) pvd_date; 
 
   memset (_pvd_date, '0', 16);
-  _pvd_date[16] = (int8_t) 0; /* tz */
+  _pvd_date[16] = (iso712_t) 0; /* Start out with time zone GMT. */
 
-  if (!_tm) return;
+  if (!p_tm) return;
 
   snprintf(_pvd_date, 17, 
            "%4.4d%2.2d%2.2d" "%2.2d%2.2d%2.2d" "%2.2d",
-           _tm->tm_year + 1900, _tm->tm_mon + 1, _tm->tm_mday,
-           _tm->tm_hour, _tm->tm_min, _tm->tm_sec,
+           p_tm->tm_year + 1900, p_tm->tm_mon + 1, p_tm->tm_mday,
+           p_tm->tm_hour, p_tm->tm_min, p_tm->tm_sec,
            0 /* 1/100 secs */ );
-  
-  _pvd_date[16] = (int8_t) 0; /* tz */
+
+  /* Adjust for daylight savings time. */
+  if (p_tm->tm_isdst > 0) _pvd_date[16] -= (iso712_t) 4;
+
+#ifdef HAVE_TM_GMTOFF
+  /* Set time zone in 15-minute interval encoding. */
+  _pvd_date[16] -= p_tm->tm_gmtoff / (15 * 60);
+  if (_pvd_date[16] < -48 ) {
+    
+    cdio_warn ("Converted ISO 9660 timezone %d is less than -48. Adjusted", 
+               (int) _pvd_date[16]);
+    _pvd_date[16] = -48;
+  } else if (_pvd_date[16] > 52) {
+    cdio_warn ("Converted ISO 9660 timezone %d is over 52. Adjusted", 
+               (int) _pvd_date[16]);
+    _pvd_date[16] = 52;
+  }
+#endif
+
 }
 
 /*!
-   Convert ISO-9660 file name that stored in a directory entry into 
-   what's usually listed as the file name in a listing.
-   Lowercase name, and remove trailing ;1's or .;1's and
-   turn the other ;'s into version numbers.
+   Convert an ISO-9660 file name which is in the format usually stored
+   in a ISO 9660 directory entry into what's usually listed as the
+   file name in a listing.  Lowercase name, and remove trailing ;1's
+   or .;1's and turn the other ;'s into version numbers.
 
-   The length of the translated string is returned.
+   @param psz_oldname the ISO-9660 filename to be translated.
+   @param psz_newname returned string. The caller allocates this and
+   it should be at least the size of psz_oldname.
+   @return length of the translated string is returned. It will be no greater
+   than the length of psz_oldname.
 */
 int 
-iso9660_name_translate(const char *psz_old, char *psz_new)
+iso9660_name_translate(const char *psz_oldname, char *psz_newname)
 {
-  return iso9660_name_translate_ext(psz_old, psz_new, 0);
+  return iso9660_name_translate_ext(psz_oldname, psz_newname, 0);
 }
 
 /*!
-   Convert ISO-9660 file name that stored in a directory entry into
-   what's usually listed as the file name in a listing.  Lowercase
-   name if not using Joliet extension. Remove trailing ;1's or .;1's and
-   turn the other ;'s into version numbers.
+   Convert an ISO-9660 file name which is in the format usually stored
+   in a ISO 9660 directory entry into what's usually listed as the
+   file name in a listing.  Lowercase name if no Joliet Extension
+   interpretation. Remove trailing ;1's or .;1's and turn the other
+   ;'s into version numbers.
 
-   The length of the translated string is returned.
+   @param psz_oldname the ISO-9660 filename to be translated.
+   @param psz_newname returned string. The caller allocates this and
+   it should be at least the size of psz_oldname.
+   @param i_joliet_level 0 if not using Joliet Extension. Otherwise the
+   Joliet level.
+   @return length of the translated string is returned. It will be no greater
+   than the length of psz_oldname.
 */
 int 
 iso9660_name_translate_ext(const char *psz_oldname, char *psz_newname, 
@@ -374,7 +402,7 @@ iso9660_strncpy_pad(char dst[], const char src[], size_t len,
 
     case ISO9660_ACHARS:
       for (idx = 0; src[idx]; idx++)
-        if (!iso9660_isachar (src[idx]))
+        if (!iso9660_is_achar (src[idx]))
           {
             cdio_warn ("string '%s' fails a-character constraint (pos = %d)",
                       src, idx);
@@ -384,7 +412,7 @@ iso9660_strncpy_pad(char dst[], const char src[], size_t len,
 
     case ISO9660_DCHARS:
       for (idx = 0; src[idx]; idx++)
-        if (!iso9660_isdchar (src[idx]))
+        if (!iso9660_is_dchar (src[idx]))
           {
             cdio_warn ("string '%s' fails d-character constraint (pos = %d)",
                       src, idx);
@@ -415,7 +443,7 @@ iso9660_strncpy_pad(char dst[], const char src[], size_t len,
    underscore.
 */
 bool
-iso9660_isdchar (int c)
+iso9660_is_dchar (int c)
 {
   if (!IN (c, 0x30, 0x5f)
       || IN (c, 0x3a, 0x40)
@@ -432,7 +460,7 @@ iso9660_isdchar (int c)
    symbol.   
 */
 bool
-iso9660_isachar (int c)
+iso9660_is_achar (int c)
 {
   if (!IN (c, 0x20, 0x5f)
       || IN (c, 0x23, 0x24)
@@ -487,7 +515,8 @@ iso9660_set_pvd(void *pd,
   memset(&ipd,0,sizeof(ipd)); /* paranoia? */
 
   /* magic stuff ... thatis CD XA marker... */
-  strcpy(((char*)&ipd)+ISO_XA_MARKER_OFFSET, ISO_XA_MARKER_STRING);
+  strncpy(((char*)&ipd)+ISO_XA_MARKER_OFFSET, ISO_XA_MARKER_STRING,
+          sizeof(ISO_XA_MARKER_STRING));
 
   ipd.type = to_711(ISO_VD_PRIMARY);
   iso9660_strncpy_pad (ipd.id, ISO_STANDARD_ID, 5, ISO9660_DCHARS);
@@ -684,6 +713,22 @@ iso9660_pathtable_init (void *pt)
   memset (pt, 0, ISO_BLOCKSIZE); /* fixme */
 }
 
+/*!
+  Returns POSIX mode bitstring for a given file.
+*/
+mode_t 
+iso9660_get_posix_filemode(const iso9660_stat_t *p_iso_dirent) 
+{
+  mode_t mode = 0;
+
+  if (yep == p_iso_dirent->rr.b3_rock) {
+    return iso9660_get_posix_filemode_from_rock(&p_iso_dirent->rr);
+  } else if (p_iso_dirent->b_xa) {
+    return iso9660_get_posix_filemode_from_xa(p_iso_dirent->xa.attributes);
+  }
+  return mode;
+}
+
 static const iso_path_table_t *
 pathtable_get_entry (const void *pt, unsigned int entrynum)
 {
@@ -844,7 +889,7 @@ iso9660_dirname_valid_p (const char pathname[])
   
   len = 0;
   for (; *p; p++)
-    if (iso9660_isdchar (*p))
+    if (iso9660_is_dchar (*p))
       {
         len++;
         if (len > 8)
@@ -910,7 +955,7 @@ iso9660_pathname_valid_p (const char pathname[])
     int dots = 0;
 
     for (; *p; p++)
-      if (iso9660_isdchar (*p))
+      if (iso9660_is_dchar (*p))
         {
           len++;
           if (dots == 0 ? len > 8 : len > 3)
