@@ -1,8 +1,8 @@
 /*
-    $Id: iso9660_fs.c,v 1.30 2005/09/15 06:36:01 rocky Exp $
+    $Id: iso9660_fs.c,v 1.35 2006/03/17 22:36:31 rocky Exp $
 
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
-    Copyright (C) 2003, 2004, 2005 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2003, 2004, 2005, 2006 Rocky Bernstein <rocky@panix.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,8 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301 USA.
 */
 /* iso9660 filesystem-based routines */
 
@@ -52,7 +53,7 @@
 
 #include <stdio.h>
 
-static const char _rcsid[] = "$Id: iso9660_fs.c,v 1.30 2005/09/15 06:36:01 rocky Exp $";
+static const char _rcsid[] = "$Id: iso9660_fs.c,v 1.35 2006/03/17 22:36:31 rocky Exp $";
 
 /* Implementation of iso9660_t type */
 struct _iso9660_s {
@@ -163,7 +164,7 @@ adjust_fuzzy_pvd( iso9660_t *p_iso )
   Open an ISO 9660 image for reading in either fuzzy mode or not.
 */
 static iso9660_t *
-iso9660_open_ext_private (const char *pathname,
+iso9660_open_ext_private (const char *psz_path,
 			  iso_extension_mask_t iso_extension_mask,
 			  uint16_t i_fuzz, bool b_fuzzy)
 {
@@ -172,7 +173,7 @@ iso9660_open_ext_private (const char *pathname,
 
   if (!p_iso) return NULL;
   
-  p_iso->stream = cdio_stdio_new( pathname );
+  p_iso->stream = cdio_stdio_new( psz_path );
   if (NULL == p_iso->stream) 
     goto error;
 
@@ -204,9 +205,9 @@ iso9660_open_ext_private (const char *pathname,
   a mode. NULL is returned on error.
 */
 iso9660_t *
-iso9660_open (const char *pathname /*, mode*/)
+iso9660_open (const char *psz_path /*, mode*/)
 {
-  return iso9660_open_ext(pathname, ISO_EXTENSION_NONE);
+  return iso9660_open_ext(psz_path, ISO_EXTENSION_NONE);
 }
 
 /*!
@@ -718,7 +719,7 @@ iso9660_fs_read_superblock (CdIo_t *p_cdio,
       cdio_read_data_sectors ( p_cdio, buf, ISO_PVD_SECTOR+1, ISO_BLOCKSIZE,
 			       1 );
 
-  if (DRIVER_OP_SUCCESS == driver_return) {
+    if (DRIVER_OP_SUCCESS == driver_return) {
       /* The size of a PVD or SVD is smaller than a sector. So we
 	 allocated a bigger block above (buf) and now we'll copy just
 	 the part we need to save.
@@ -812,6 +813,7 @@ _iso9660_dir_to_statbuf (iso9660_dir_t *p_iso9660_dir, bool_3way_t b_xa,
   p_stat->size    = from_733 (p_iso9660_dir->size);
   p_stat->secsize = _cdio_len2blocks (p_stat->size, ISO_BLOCKSIZE);
   p_stat->rr.b3_rock = dunno; /*FIXME should do based on mask */
+  p_stat->b_xa    = false; 
 
   {
     char rr_fname[256] = "";
@@ -835,9 +837,9 @@ _iso9660_dir_to_statbuf (iso9660_dir_t *p_iso9660_dir, bool_3way_t b_xa,
       strncpy(p_stat->filename, rr_fname, i_rr_fname+1);
     } else {
       if ('\0' == p_iso9660_dir->filename[0] && 1 == i_fname)
-	strcpy (p_stat->filename, ".");
+	strncpy (p_stat->filename, ".", sizeof("."));
       else if ('\1' == p_iso9660_dir->filename[0] && 1 == i_fname)
-	strcpy (p_stat->filename, "..");
+	strncpy (p_stat->filename, "..", sizeof(".."));
 #ifdef HAVE_JOLIET
       else if (i_joliet_level) {
 	int i_inlen = i_fname;
@@ -899,7 +901,8 @@ _iso9660_dir_to_statbuf (iso9660_dir_t *p_iso9660_dir, bool_3way_t b_xa,
 		      xa_data->signature[0], xa_data->signature[1]);
 	  return p_stat;
 	}
-      p_stat->xa = *xa_data;
+      p_stat->b_xa = true;
+      p_stat->xa   = *xa_data;
     }
   }
   return p_stat;
@@ -924,9 +927,9 @@ iso9660_dir_to_name (const iso9660_dir_t *iso9660_dir)
   /* (iso9660_dir->file_flags & ISO_DIRECTORY) */
   
   if (iso9660_dir->filename[0] == '\0')
-    strcpy (namebuf, ".");
+    strncpy (namebuf, ".", sizeof("."));
   else if (iso9660_dir->filename[0] == '\1')
-    strcpy (namebuf, "..");
+    strncpy (namebuf, "..", sizeof(".."));
   else
     strncpy (namebuf, iso9660_dir->filename, iso9660_dir->filename_len);
 
@@ -985,7 +988,7 @@ _fs_stat_root (CdIo_t *p_cdio)
 }
 
 static iso9660_stat_t *
-_fs_stat_iso_root (iso9660_t *p_iso)
+_ifs_stat_root (iso9660_t *p_iso)
 {
   iso9660_stat_t *p_stat;
   iso9660_dir_t *p_iso9660_dir;
@@ -1186,7 +1189,7 @@ _fs_iso_stat_traverse (iso9660_t *p_iso, const iso9660_stat_t *_root,
 }
 
 /*!
-  Get file status for pathname into stat. NULL is returned on error.
+  Get file status for psz_path into stat. NULL is returned on error.
  */
 iso9660_stat_t *
 iso9660_fs_stat (CdIo_t *p_cdio, const char psz_path[])
@@ -1211,7 +1214,7 @@ iso9660_fs_stat (CdIo_t *p_cdio, const char psz_path[])
 }
 
 /*!
-  Get file status for pathname into stat. NULL is returned on error.
+  Get file status for psz_path into stat. NULL is returned on error.
   pathname version numbers in the ISO 9660
   name are dropped, i.e. ;1 is removed and if level 1 ISO-9660 names
   are lowercased.
@@ -1239,7 +1242,7 @@ iso9660_fs_stat_translate (CdIo_t *p_cdio, const char psz_path[],
 }
 
 /*!
-  Get file status for pathname into stat. NULL is returned on error.
+  Get file status for psz_path into stat. NULL is returned on error.
  */
 iso9660_stat_t *
 iso9660_ifs_stat (iso9660_t *p_iso, const char psz_path[])
@@ -1251,7 +1254,7 @@ iso9660_ifs_stat (iso9660_t *p_iso, const char psz_path[])
   if (!p_iso)    return NULL;
   if (!psz_path) return NULL;
 
-  p_root = _fs_stat_iso_root (p_iso);
+  p_root = _ifs_stat_root (p_iso);
   if (!p_root) return NULL;
 
   splitpath = _cdio_strsplit (psz_path, '/');
@@ -1263,7 +1266,7 @@ iso9660_ifs_stat (iso9660_t *p_iso, const char psz_path[])
 }
 
 /*!
-  Get file status for pathname into stat. NULL is returned on error.
+  Get file status for psz_path into stat. NULL is returned on error.
   pathname version numbers in the ISO 9660
   name are dropped, i.e. ;1 is removed and if level 1 ISO-9660 names
   are lowercased.
@@ -1278,7 +1281,7 @@ iso9660_ifs_stat_translate (iso9660_t *p_iso, const char psz_path[])
   if (!p_iso)    return NULL;
   if (!psz_path) return NULL;
 
-  p_root = _fs_stat_iso_root (p_iso);
+  p_root = _ifs_stat_root (p_iso);
   if (NULL == p_root) return NULL;
 
   p_psz_splitpath = _cdio_strsplit (psz_path, '/');
@@ -1290,8 +1293,10 @@ iso9660_ifs_stat_translate (iso9660_t *p_iso, const char psz_path[])
 }
 
 /*! 
-  Read pathname (a directory) and return a list of iso9660_stat_t
+  Read psz_path (a directory) and return a list of iso9660_stat_t
   of the files inside that. The caller must free the returned result.
+
+  b_mode2 is historical. It is not used.
 */
 CdioList_t * 
 iso9660_fs_readdir (CdIo_t *p_cdio, const char psz_path[], bool b_mode2)
@@ -1356,7 +1361,7 @@ iso9660_fs_readdir (CdIo_t *p_cdio, const char psz_path[], bool b_mode2)
 }
 
 /*! 
-  Read pathname (a directory) and return a list of iso9660_stat_t
+  Read psz_path (a directory) and return a list of iso9660_stat_t
   of the files inside that. The caller must free the returned result.
 */
 CdioList_t * 
@@ -1428,9 +1433,9 @@ iso9660_ifs_readdir (iso9660_t *p_iso, const char psz_path[])
 }
 
 static iso9660_stat_t *
-find_fs_lsn_recurse (CdIo_t *p_cdio, const char pathname[], lsn_t lsn)
+find_fs_lsn_recurse (CdIo_t *p_cdio, const char psz_path[], lsn_t lsn)
 {
-  CdioList_t *entlist = iso9660_fs_readdir (p_cdio, pathname, true);
+  CdioList_t *entlist = iso9660_fs_readdir (p_cdio, psz_path, true);
   CdioList_t *dirlist =  _cdio_list_new ();
   CdioListNode_t *entnode;
     
@@ -1444,7 +1449,7 @@ find_fs_lsn_recurse (CdIo_t *p_cdio, const char pathname[], lsn_t lsn)
       char _fullname[4096] = { 0, };
       char *filename = (char *) statbuf->filename;
 
-      snprintf (_fullname, sizeof (_fullname), "%s%s/", pathname, filename);
+      snprintf (_fullname, sizeof (_fullname), "%s%s/", psz_path, filename);
 
       if (statbuf->type == _STAT_DIR
           && strcmp ((char *) statbuf->filename, ".") 
@@ -1481,6 +1486,60 @@ find_fs_lsn_recurse (CdIo_t *p_cdio, const char pathname[], lsn_t lsn)
   return NULL;
 }
 
+static iso9660_stat_t *
+find_ifs_lsn_recurse (iso9660_t *p_iso, const char psz_path[], lsn_t lsn)
+{
+  CdioList_t *entlist = iso9660_ifs_readdir (p_iso, psz_path);
+  CdioList_t *dirlist =  _cdio_list_new ();
+  CdioListNode_t *entnode;
+    
+  cdio_assert (entlist != NULL);
+
+  /* iterate over each entry in the directory */
+  
+  _CDIO_LIST_FOREACH (entnode, entlist)
+    {
+      iso9660_stat_t *statbuf = _cdio_list_node_data (entnode);
+      char _fullname[4096] = { 0, };
+      char *filename = (char *) statbuf->filename;
+
+      snprintf (_fullname, sizeof (_fullname), "%s%s/", psz_path, filename);
+
+      if (statbuf->type == _STAT_DIR
+          && strcmp ((char *) statbuf->filename, ".") 
+          && strcmp ((char *) statbuf->filename, ".."))
+        _cdio_list_append (dirlist, strdup (_fullname));
+
+      if (statbuf->lsn == lsn) {
+	unsigned int len=sizeof(iso9660_stat_t)+strlen(statbuf->filename)+1;
+	iso9660_stat_t *ret_stat = calloc(1, len);
+	memcpy(ret_stat, statbuf, len);
+        _cdio_list_free (entlist, true);
+        _cdio_list_free (dirlist, true);
+        return ret_stat;
+      }
+      
+    }
+
+  _cdio_list_free (entlist, true);
+
+  /* now recurse/descend over directories encountered */
+
+  _CDIO_LIST_FOREACH (entnode, dirlist)
+    {
+      char *_fullname = _cdio_list_node_data (entnode);
+      iso9660_stat_t *ret_stat = find_ifs_lsn_recurse (p_iso, _fullname, lsn);
+
+      if (NULL != ret_stat) {
+        _cdio_list_free (dirlist, true);
+        return ret_stat;
+      }
+    }
+
+  _cdio_list_free (dirlist, true);
+  return NULL;
+}
+
 /*!
    Given a directory pointer, find the filesystem entry that contains
    lsn and return information about it.
@@ -1488,11 +1547,23 @@ find_fs_lsn_recurse (CdIo_t *p_cdio, const char pathname[], lsn_t lsn)
    Returns stat_t of entry if we found lsn, or NULL otherwise.
  */
 iso9660_stat_t *
-iso9660_find_fs_lsn(CdIo_t *p_cdio, lsn_t i_lsn)
+iso9660_fs_find_lsn(CdIo_t *p_cdio, lsn_t i_lsn)
 {
   return find_fs_lsn_recurse (p_cdio, "/", i_lsn);
 }
 
+/*!
+   Given a directory pointer, find the filesystem entry that contains
+   lsn and return information about it.
+
+   Returns stat_t of entry if we found lsn, or NULL otherwise.
+ */
+iso9660_stat_t *
+iso9660_ifs_find_lsn(iso9660_t *p_iso, lsn_t i_lsn)
+{
+  return find_ifs_lsn_recurse (p_iso, "/", i_lsn);
+}
+
 /*!
   Return true if ISO 9660 image has extended attrributes (XA).
 */
