@@ -1,5 +1,5 @@
 /*
-    $Id: p_block.c,v 1.8 2005/09/20 00:42:14 rocky Exp $
+    $Id: p_block.c,v 1.10 2006/01/05 21:58:52 rocky Exp $
 
     Copyright (C) 2004, 2005 Rocky Bernstein <rocky@panix.com>
     Copyright (C) 1998 Monty xiphmont@mit.edu
@@ -166,7 +166,7 @@ i_vfragment_constructor(void)
   return(ret);
 }
 
-static inline void 
+static void 
 i_v_fragment_destructor(v_fragment_t *v)
 {
   free(v);
@@ -188,6 +188,13 @@ new_v_fragment(cdrom_paranoia_t *p, c_block_t *one,
   b->size=end-begin;
   b->lastsector=last;
 
+#if TRACE_PARANOIA
+  fprintf(stderr, "- Verified [%ld-%ld] (0x%04X...0x%04X)%s\n",
+	  begin, end,
+	  b->vector[0]&0xFFFF, b->vector[b->size-1]&0xFFFF,
+	  last ? " *" : "");
+#endif
+
   return(b);
 }
 
