@@ -1,7 +1,7 @@
 /*
-    $Id: cdda-player.c,v 1.36 2005/09/15 06:36:01 rocky Exp $
+    $Id: cdda-player.c,v 1.41 2006/03/17 19:36:54 rocky Exp $
 
-    Copyright (C) 2005 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2005, 2006 Rocky Bernstein <rocky@panix.com>
 
     Adapted from Gerd Knorr's player.c program  <kraxel@bytesex.org>
     Copyright (C) 1997, 1998 
@@ -18,7 +18,8 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301 USA.
 */
 
 #ifdef HAVE_CONFIG_H
@@ -49,10 +50,6 @@
 
 #include <signal.h>
 
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
-#endif
-
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
@@ -74,6 +71,7 @@
 #include <cdio/logging.h>
 
 #include "cddb.h"
+#include "getopt.h"
 
 static bool play_track(track_t t1, track_t t2);
 static void display_cdinfo(CdIo_t *p_cdio, track_t i_tracks, 
@@ -105,15 +103,15 @@ int                i_vol_port   = -1; /* If -1 get retrieve volume port.
 					 of a working volume port.
 				       */
 
-bool               b_cd         = false;
-bool               auto_mode    = false;
-bool               b_verbose    = false;
-bool               debug        = false;
-bool               interactive  = true;
+bool               b_cd            = false;
+bool               auto_mode       = false;
+bool               b_verbose       = false;
+bool               debug           = false;
+bool               b_interactive   = true;
 bool               b_prefer_cdtext = true; 
-bool               b_cddb   = false; /* CDDB database present */
-bool               b_db     = false; /* we have a database at all */
-bool               b_record = false; /* we have a record for
+bool               b_cddb          = false; /* CDDB database present */
+bool               b_db            = false; /* we have a database at all */
+bool               b_record        = false; /* we have a record for
 					the inserted CD */
 bool               b_all_tracks = false; /* True if we display all tracks*/
 
@@ -198,7 +196,7 @@ typedef enum {
 static void
 tty_raw()
 {
-  if (!interactive) return;
+  if (!b_interactive) return;
   
   initscr();
   cbreak();
@@ -213,7 +211,7 @@ tty_raw()
 static void
 tty_restore()
 {
-  if (!interactive) return;
+  if (!b_interactive) return;
   endwin();
 }
 
@@ -243,7 +241,7 @@ select_wait(int sec)
 static void 
 action(const char *psz_action)
 {
-  if (!interactive) {
+  if (!b_interactive) {
     if (b_verbose && psz_action)
       fprintf(stderr,"action: %s\n", psz_action);
     return;
@@ -257,12 +255,13 @@ action(const char *psz_action)
   refresh();
 }
 
-static void inline
+
+static void 
 xperror(const char *psz_msg)
 {
   char line[80];
   
-  if (!interactive) {
+  if (!b_interactive) {
     if (b_verbose) {
       fprintf(stderr, "error: ");
       perror(psz_msg);
@@ -283,7 +282,7 @@ xperror(const char *psz_msg)
 static void 
 finish(const char *psz_msg, int rc)
 {
-  if (interactive) {
+  if (b_interactive) {
     mvprintw(LINE_LAST, 0, (char *) "%s, exiting...\n", psz_msg);
     clrtoeol();
     refresh();
@@ -379,6 +378,31 @@ read_subchannel(CdIo_t *p_cdio)
   return b_ok;
 }
 
+#ifdef HAVE_CDDB
+/*! This routine is called by vcd routines on error. 
+   Setup is done by init_input_plugin.
+*/
+static void 
+cddb_log_handler (cddb_log_level_t level, const char message[])
+{
+  switch (level) {
+  case CDDB_LOG_DEBUG:
+  case CDDB_LOG_INFO:
+    if (!b_verbose)
+      return;
+    /* Fall through if to warn case */
+  case CDDB_LOG_WARN:
+  case CDDB_LOG_ERROR:
+  case CDDB_LOG_CRITICAL:
+  default:
+    xperror(message);
+    break;
+  }
+  
+  /* gl_default_cdio_log_handler (level, message); */
+}
+#endif /* HAVE_CDDB  */
+
 static void 
 get_cddb_disc_info(CdIo_t *p_cdio) 
 {
@@ -393,7 +417,7 @@ get_cddb_disc_info(CdIo_t *p_cdio)
     cddb_disc_set_genre(p_cddb_disc, genre);
     cddb_disc_set_year(p_cddb_disc, i_year);
   }
-#endif
+#endif /* HAVE_CDDB */
   return;
 }
 
@@ -595,7 +619,7 @@ display_status(bool b_status_only)
 {
   char line[80];
 
-  if (!interactive) return;
+  if (!b_interactive) return;
 
   if (!b_cd) {
     sprintf(line,"no CD in drive (%s)", psz_device);
@@ -714,7 +738,7 @@ get_cdtext_track_info(track_t i_track)
 
   if (NULL != p_cdtext) {
     add_cdtext_track_info("%s", title, CDTEXT_TITLE);
-    add_cdtext_track_info("%s", title, CDTEXT_PERFORMER);
+    add_cdtext_track_info("%s", artist, CDTEXT_PERFORMER);
     cdtext_destroy(p_cdtext);
   }
 }
@@ -746,7 +770,7 @@ display_cdinfo(CdIo_t *p_cdio, track_t i_tracks, track_t i_first_track)
   int len;
   char line[80];
   
-  if (!interactive) return;
+  if (!b_interactive) return;
 
   if (!b_cd) sprintf(line, "-");
   else {
@@ -857,7 +881,7 @@ display_tracks(void)
   if (b_record) {
     i_line=LINE_TRACK_PREV - 1;
     for (i = i_first_track; i <= i_last_track; i++) {
-      char line[80];
+      char line[200]="";
       s = cdio_audio_get_msf_seconds(&toc[i+1]) 
 	- cdio_audio_get_msf_seconds(&toc[i]);
       read_subchannel(p_cdio);
@@ -1182,8 +1206,17 @@ ps_list_tracks(void)
     
     printf("labelfont setfont\n");
     printf("120 %d moveto (%d) show\n", y, i);
-    printf("150 %d moveto (%s) show\n", y, cd_info[i].title);
-    
+    {
+      char line[200]="";
+      if ( strlen(cd_info[i].title) )
+	strcat(line, cd_info[i].title);
+      if ( strlen(cd_info[i].artist) > 0 ) {
+	if (strlen(cd_info[i].title))
+	  strcat(line, " / ");
+	strcat(line, cd_info[i].artist);
+      }
+      printf("150 %d moveto (%s) show\n", y, line);
+    }
     printf("timefont setfont\n");
     printf("420 %d moveto (%2d:%02d) show\n", y, 
 	   s / CDIO_CD_SECS_PER_MIN, s % CDIO_CD_SECS_PER_MIN);
@@ -1282,35 +1315,35 @@ main(int argc, char *argv[])
 	stop_track = start_track+1;
 	one_track = 1;
       }
-      interactive = false;
+      b_interactive = false;
       cd_op = PLAY_TRACK;
       break;
     case 'p':
-      interactive = false;
+      b_interactive = false;
       cd_op = PLAY_CD;
       break;
     case 'l':
-      interactive = false;
+      b_interactive = false;
       cd_op = LIST_TRACKS;
       break;
     case 'C':
-      interactive = false;
+      b_interactive = false;
       cd_op = CLOSE_CD;
       break;
     case 'c':
-      interactive = false;
+      b_interactive = false;
       cd_op = PS_LIST_TRACKS;
       break;
     case 's':
-      interactive = false;
+      b_interactive = false;
       cd_op = STOP_PLAYING;
       break;
     case 'S':
-      interactive = false;
+      b_interactive = false;
       cd_op = LIST_SUBCHANNEL;
       break;
     case 'e':
-      interactive = false;
+      b_interactive = false;
       cd_op = EJECT_CD;
       break;
     case 'k':
@@ -1346,7 +1379,7 @@ main(int argc, char *argv[])
     cdio_free_device_list(ppsz_cdda_drives);
   }
   
-  if (!interactive) {
+  if (!b_interactive) {
     b_sig = true;
     nostop=1;
   }
@@ -1371,7 +1404,13 @@ main(int argc, char *argv[])
       fprintf(stderr,"ok\n");
   }
   
-  if (!interactive) {
+  if (b_interactive) {
+#ifdef HAVE_CDDB
+    cddb_log_set_handler (cddb_log_handler);
+#else
+    ;
+#endif
+  }  else {
     b_sig = true;
     nostop=1;
     if (EJECT_CD == cd_op) {
@@ -1388,6 +1427,7 @@ main(int argc, char *argv[])
       if (p_cdio)
 	switch (cd_op) {
 	case STOP_PLAYING:
+	  b_cd = true;
 	  i_rc = cd_stop(p_cdio) ? 0 : 1;
 	  break;
 	case EJECT_CD:
